% demo10a -- Illustrates use of correlation analysis and cross-correlation function to 
% estimate the lagged response of a linear dynamic system in which:
%  1-- the input series u(t) is autocorrelated (AR(1) process) 
%  2-- the the output y(t) is a weighted average of the input at times t and t-1 (causal system),
%       plus a noise component made up of random normal noise
%  3-- You controls:
%       * the length of time series
%       * the autocorrelation of the input (by specifying the AR coefficent)
%       * the fractional weighting of importance on input at times t and t-1 to output at time t
%       * the signal/noise ratio, by specifying the ratio of standard deviations of the 
%           noise to input  output y
%
% An autocorrelated input time series, u(t), is simulated using an AR(1) model
%       with specified AR(1) coefficient; u(t) is scaled to mean 0 and
%       variance 1
% A synthetic output series is generated by adding random normal noise e(t) to u(t).  The
%       noise variance is set by specifying the ratio of the standard deviations of 
%       e(t) and u(t).  This procedure is equivalent to specifying that
%       there is NO lag in the response of y to u:  the response is
%       confined to lag 0, but the signal is contaminated by noise
% The two series (input u(t) and output y(t)) are re-adjusted to have means
%       exactly 0
% The exercise is aimed at showing how given the data we can identify the
%       lag in response (in this case, no lag)
% Matlab function cra is used to generate and plot various relevant quantities
%   -autocovariance functions of u and y
%   -cross covariance function of u to y
%   -impulse response function of system with input u and output y
% Cra is called twice, to illustrate the importance of the step of prewhitening the input
%   1- with no prewhitening (ccf is just that between the original series)
%   2- with AR(10) prewhitening of input and filtering of output
%
% Additional purposes of this demo are to illustrate: 
%
% importance of signal/noise ratio in being able to identify
% lagged resonse
%
% change in appearance of time series with change in specified
% autoregressive (first order) order
%
% autocovariance, cross-correlation and impulse
% response functions
%
% function quenouille; given two time series and a specified number of
% lags, computes the ccf and its 95%, 99% and 99.9% CI (two tailed test)
% using Quenouille's equaton. 
%
% 

clear;
close all;
clc;



%--- PROMPT FOR SETTINGS

prompt={'Enter length of time series (N>30):',...
    'Enter AR(1)coefficient of model to autocorrelate input ( -1< a1 <=0):',...
    'Enter the decimal proportion of response for lag t-1 (0<=p<=1)',...
    'Enter ratio of standard deviations of noise to input (0<=f<=10):'};
name='Settings for simulation';
numlines=1;
defaultanswer={'90','-0.7','0.4','0.50'};

answer=inputdlg(prompt,name,numlines,defaultanswer);

% Sample size
N=str2num(answer{1});
N=floor(N);
if N<30
    error('Sample size N must be at least 30');
end;

% First-order autocorrelation coefficient of input model ;  note that negative coefficient goes with positive autocorrelation 
% (Matlab system identification toolbox convention)
ar_coef=str2num(answer{2});  % coef of AR(1) model used to simulate input time series
if ar_coef >0 | ar_coef<=-1
    error('AR(1) coef must be between 0 and -1; can be 0 but must be greater than -1');
end;

% Decimal proportion of dependence on output at time t on input at time t-1 
p=str2num(answer{3});  % proportion of lagged response applied to lag t-1 (1-p is applied to lag 0)
if ~(p>=0 & p<=1);
    error('p must be between 0 and 1, inclusive');
end;

% Ratio of standard deviation of noise to standard deviation of input; this ration specifies signal/noise ratio
noise_ratio= str2num(answer{4}); % ratio of standard deviations of noise to autocorrelated input to be
%    used in generating the output;  random normal noise is added to the
%    input to form the output.  Must be greater than 0. 
if noise_ratio<0 | noise_ratio>10;
    error('Noise ratio must be in range [0 10], exclusive');
end;

strN = ['Time series length N = ' num2str(N)];
strar=['u simulated with random normal noise and AR(1) operator [1 ' num2str(ar_coef) ']'];
strp =['y generated by:  y= ' num2str(p) ' * u(t-1) + ' num2str(1-p) ' * u(t) + V(t)'];
strnoise=['V is random normal noise scaled such that std(V)/std(u) = ' num2str(noise_ratio)]; 



%--- GENERATE A SIMULATED AUTOCORRELATED INPUT SERIES, LENGTH n WITH AN AR(1) MODEL

t=(1:(N+1))'; % time vector, length allows for 1 leading value needed for first lagged response on t-1

m = idpoly([1 ar_coef],[]);  % Specify the AR model for simulating input;  m is a model object (see help idpoly)

u1 = normrnd(0,1,length(t),1); %  sample random normal noise
u1=u1-mean(u1); % re-scale the sampled noise to have mean exactly zero
dat = iddata(u1,[],1);  % convert the time series u1 to an "iddata" object (see help iddata)

u=sim(m,dat.OutputData); % using the sample of zero-mean random noise, simulate an AR(1) time series;
% Note that first order autocorrelation cofficient of this simulation will exactly equal specified ar_coef 
% because of random variation

% Rescale the autocorrelated input to variance 1.0 and mean exactly 0;
umean=mean(u);
udepart=u-umean;
u = udepart/std(udepart);



%--- GENERATE SIMULATED OUTPUT, y(t)

%-- Generate synthetic output series as weighted average of input series at times t and t-1 
% plus plus random normal noise; the random noise is sampled from normal distribution with mean 0 
% and standard deviation equal to the specified noise_ratio. Recall that the input series u(t)
% has been scaled to have a standard deviation of 1.0.  Thus specifying the standard deviation 
% of the noise is equivalent to specifying the ratio of standard deviations of noise to input

y =   p*u(1:(end-1)) + (1-p)*u(2:end) + normrnd(0,noise_ratio,N,1); 
t=t(2:end);
u=u(2:end);

%--- Adjust u and y to exactly mean 0;
u=u-mean(u);
y-y-mean(y);


%--- TIME SERIES PLOT OF INPUT AND OUTPUT

tlims=[t(1)-1 t(end)+1];

% Compute correlation coefficient between u and y
r=corrcoef(u,y);
r=r(1,2);

% Plot z-scores of u and y, with correlation coefficient and other model information annotated in title
figure(1);
rstr=['Time series of autocorrelated input (u) and output (y), r = ' num2str(r)];
titlestr={rstr,strar,strp,strnoise};
if length(u)<100
    hp1=plot(t,zscore(u),'-o',t,zscore(y),'-^',[t(1) t(end)],[0 0]);
else
     hp1=plot(t,zscore(u),t,zscore(y),[t(1) t(end)],[0 0]);
end
set(gca,'XLim',tlims)
set(hp1(1),'LineWidth',2);
set(hp1(2),'Linewidth',2);
xlabel('time')
ylabel('z-scores of time series (dimensionless)');
legend('u','y');
title(titlestr);
grid on;
set(gca,'Position',[ 0.1300    0.1100    0.7750    0.6590]);


%--- APPLY SYSTEM IDENTIFICATION FUNCTION CRA FOR IMPULSE RESPONSE ANALYSIS BY CORRELATION METHOD

dat =[y u]; % y is output, u input

% Aside to call to get ccf and 99% CI by quenouille function
K=10;
[rq,pq,rcq,Nq,kq]=quenouille(u,y,K);


% First,  look at acvf's and ccf if no prewhitening of input
na=0; % order for prewhitening model
figure(2);
m=10; % max lag for ccf plot
cra(dat,m,na,2); % "2" specifies want all 4 plots (acvf's as well as ccf and irf)

% Add markers to plot
hf=get(gcf,'Children'); %  handles to axes
for j=1:4
    hathis=hf(j); 
    set(hathis,'Xgrid','on')
    hline=findobj(hathis,'Type','Line');
    for k=length(hline)
       set(hline(k),'Marker','o');
    end;
    
   
    
end;
   
legend('IRF','99% CI')

% Add Quenouille CIs to cross-correlation plot
axes(hf(2))
 hline1=line(xlim,[rcq(2) rcq(2)]);
 hline2=line(xlim,[-rcq(2) -rcq(2)]);
 set (hline1,'Color',[1 0 0],'LineStyle','--')
set (hline2,'Color',[1 0 0],'LineStyle','--')
legend('ccf','99% CI')


% Second,  look at acvf's and ccf with prewhitening of input (by AR(10) model)
na=10; % order for prewhitening model
figure(3);
m=10; % max lag for ccf plot
cra(dat,m,na,2); % "2" specifies want all 4 plots (acvf's as well as ccf and irf)

% Add markers to polt
hf=get(gcf,'Children'); %  handles to axes
for j=1:4;
    hathis=hf(j); 
    set(hathis,'Xgrid','on')
    hline=findobj(hathis,'Type','Line');
    for k=length(hline);
       set(hline(k),'Marker','o');
    end;
   
end;
legend('IRF','99% CI')


figure(1);